@@ -214,6 +214,7 @@ typedef struct
 #define VEGAALPHA   0x09820008
 #define HPSMARTCARDKEYBOARD 0x03F01024
 #define HP_CCIDSMARTCARDKEYBOARD 0x03F00036
+#define CHICONYHPSKYLABKEYBOARD 0x04F21469
 #define KOBIL_IDTOKEN 0x0D46301D
 #define FUJITSUSMARTKEYB 0x0BF81017
 #define FEITIANR502DUAL 0x096E060D
@@ -225,9 +226,13 @@ typedef struct
 #define HID_AVIATOR	0x076B3A21
 #define HID_OMNIKEY_5422 0x076B5422
 #define HID_OMNIKEY_3X21 0x076B3031 /* OMNIKEY 3121 or 3021 or 1021 */
+#define HID_OMNIKEY_3821 0x076B3821 /* OMNIKEY 3821 */
 #define HID_OMNIKEY_6121 0x076B6632 /* OMNIKEY 6121 */
 #define CHERRY_XX44	0x046A00A7 /* Cherry Smart Terminal xx44 */
 #define FUJITSU_D323 0x0BF81024 /* Fujitsu Smartcard Reader D323 */
+#define IDENTIV_uTrust3700F		0x04E65790
+#define IDENTIV_uTrust3701F		0x04E65791
+#define IDENTIV_uTrust4701F		0x04E65724
 
 #define VENDOR_GEMALTO 0x08E6
 #define GET_VENDOR(readerID) ((readerID >> 16) & 0xFFFF)
